	
# ---------------------------ooo0ooo---------------------------
#                   .mseed to audio file converter 
#                  
#                          Ian Robinson
#                       starfishprime.co.uk
#                          April 2021
#
# converts seismic data in .mseed format to an audio file with the 
# seismic values mapped onto a frequency range
#
# Reads in either a single .mseed file or an entire directory which  
#   it then concatenates. The seismic data are mapped onto a frequency 
#   range and time may be compressed by some user defined factor
#   e.g a day compressed by a factor of 1000 results in about 80 sec of 
#   audio.
#
#   references to magFieldValues as was originally written
#          for the Earth Field magnetometer rig described in
#            https://doi.org/10.1088/1361-6552/aaacb2
#
#
#         inspired by the work of Louis Braddock Clarke
#           http://www.louisbraddockclarke.co.uk/
#
#---------------------------ooo0ooo---------------------------


# import required modules
import numpy as np
import time
import array
import multiprocessing
import obspy



from scipy import signal
from scipy.io.wavfile import write


from obspy.signal.filter import bandpass, lowpass, highpass
from obspy import UTCDateTime, read, Trace, Stream


import time
import string

from matplotlib import pyplot as plt
import matplotlib.gridspec as gridspec
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

import matplotlib.dates as mdates
import os


# tkinter used by file dialog
from tkinter import filedialog
from tkinter import *
from tkinter import filedialog

import tkinter as tk


# ---------------------------ooo0ooo---------------------------
# ---------------------------ooo0ooo---------------------------
def generateTone(frequency, length, wavFileSampleRate):
    global phi
    
    factor = (np.pi * 2.0) * float(frequency) / wavFileSampleRate
    tone = np.sin((np.arange(length) * factor) + phi)
    phi = np.arcsin(tone[-1])
    return tone
# ---------------------------ooo0ooo---------------------------
def butter_bandpass(lowcut, highCut, samplingFreq, order=5):
    nyq = 0.5 * samplingFreq
    low = lowcut / nyq
    high = highCut / nyq
    b, a = butter(order, [low, high], btype='band')
    return b, a


# ---------------------------ooo0ooo---------------------------

def butter_bandpass_filter(data, lowcut, highCut, samplingFreq, order=5):
    b, a = butter_bandpass(lowcut, highCut, samplingFreq, order=order)
    y = lfilter(b, a, data)
    return y

# ---------------------------ooo0ooo---------------------------
def ChunkIntoAveragedBlocks(data, chunkLength):
	N = len(data)
	nchuncks = N//chunkLength
	chunked= np.zeros((nchuncks), np.float32)
	
	i=0
	k=0
	
	while i<((N+1)-chunkLength):
		tempSum = 0.0
		for j in range(i, (i+chunkLength-1)):
			tempSum = tempSum + data[i]
		
		chunked[k] =tempSum/chunkLength
		k=k+1
		i=i+chunkLength
	
	return chunked

# ---------------------------ooo0ooo---------------------------
def opendataFile():
    root = tk.Tk()
    root.withdraw()
    root.filename = filedialog.askopenfilename(initialdir=os.getcwd(), title="Select file",
                                               filetypes=[("miniseed data files", "*.mseed")])
    st = read(root.filename)
    return st
# ---------------------------ooo0ooo---------------------------

def openFolder():
	root = Tk()
	root.withdraw()
	#folder_selected = filedialog.askdirectory()phaseAngle=0.0
	folder_selected = filedialog.askdirectory(initialdir=os.getcwd(), title="Select directory")
	return folder_selected

# ---------------------------ooo0ooo---------------------------
def readInFolder(resamplefreq):
	z=openFolder()
	st = Stream()
	# Read in all files within slected directory.
	listing = os.listdir(z)
	for file in listing:
		if '.mseed' in file:
			print(file)
			streamTemp=obspy.read(z+'/'+file)

			streamTemp.resample(resamplefreq) 
			st += streamTemp
	st.sort(['starttime'])
	st.merge(method=1, fill_value="interpolate")
	return st
# ---------------------------ooo0ooo---------------------------







# ---------------------------------#
#                                  #
#             Main Body            #
#                                  #
# ---------------------------------#

global phi   #starting phase of a tone (radians)

wavFileSampleRate= 44100  #: sample rate of output .wav file
speedUp=5000   #: time compression

f_resample=0.5 #: resampling freq of input mseed file
# resampling is not strictly necessary unless concatenating multiple input 
# files whose sample rates may vary.


lowCut = 0.00001 	#: filter low frequency cut-off
highCut = 0.2		#: filter high frequency cut-off

audioFmax=600.0
audioFmin=100.0
midFreq=(audioFmax+audioFmin)/2.0

audioAmplitude=10000.0
#minimum length of a single tone in output wave File
minimumToneLength=0.5




##########get data############

# either read in a folder concatenating .mseed files
#st1 = readInFolder(f_resample)

# or read in a single .mseed file
st1 = opendataFile()  # select a data file to work on
##############################

st1.resample(f_resample)
magFieldValues = st1[0].copy()  # always work with a copy

magFieldValues.detrend(type='simple') # effectivly zeroes the trace on the mean value
magFieldValues.filter('bandpass', freqmin=lowCut, freqmax=highCut, corners=4, zerophase=True)
magFieldValues.normalize()
##########get data############


# need to average and chunk input field strengths so that, after speedup, 
#  one value results in a tone at least .. say .. 0.5 sec long
# no of input values to chunk together ~ 0.5 sec * speedup * f_resample
chunkLength = int(minimumToneLength*speedUp*f_resample)
chunkedFieldValues = ChunkIntoAveragedBlocks(magFieldValues, chunkLength)

#: map the field values onto frequencies
freqValues=(chunkedFieldValues*(audioFmax-audioFmin)/2.0)+midFreq



#: determine no of wave file data points per frequency value
samplesPerTone=int(minimumToneLength*wavFileSampleRate)

#: create numpy array to store wav data
outputArray=np.zeros(len(freqValues)*samplesPerTone)


		
		
# initial phase angle for start of first tone
phi=0.0

i=0

for freq in freqValues:
	#tonewave generates a sine wave of length samplesPreTone whose
	#starting phase is the ending phase of the previous tone
	tonewave=generateTone(freq, samplesPerTone, wavFileSampleRate)
	for j in range(samplesPerTone):
		outputArray[i]=tonewave[j]
		i=i+1

# scale amplitude
outputArray=outputArray*audioAmplitude

write("output.wav", wavFileSampleRate, outputArray.astype(np.int16))


magFieldValues.plot()




# ----------------------------------------------


# ---------------------------ooo0ooo---------------------------
# ---------------------------ooo0ooo---------------------------

